// lpac_smp.cpp - shows how to use the LPAC codec API (lpac_codec_api.dll)
// (c) Tilman Liebchen
// 02.02.2001

#include <stdio.h>
#include <string.h>

#include "lpac_codec_api.h"

short main(short argc, char **argv)
{
	// predefined variables: change their values to change the sample program's behaviour
	short version = 6;				// version of the file format
	short compmode = 3;				// cpmression mode
	short joint = 1;				// joint stereo on
	short ra = 0;					// random access off
	short usage = 0;				// selects different branches of processing (see below)
	short whole = 1;				// encode/decode whole file or frame by frame
	
	// other variables
	short encode = 0, crc = 0;
	long f, frames;
	FILE *fpWav, *fpPac;
	PACINFO info;

	// evaluate command line parameters
	if (argc < 4)
	{
		printf("\nlpac_smp [e/d] [input file] [output file]\n");
		return(1);
	}

	// select encoding/decoding
	if (strcmp(argv[1], "d"))
		encode = 1;
  
	// encoding WAV -> PAC ////////////////////////////////////////////////////////////////////////
	if (encode)
	{
		if (usage == 0)
		{
			// compress file.wav to file.pac using only one function call
			EncodeWaveFile(argv[2], argv[3], version, compmode, joint, ra);
		}
		else
		{
			// encoder initialization
			EncoderCreate();

			if (usage == 1)
			{
				// files are opened by the encoder
				if (EncoderOpenFiles(argv[2], argv[3]))
				{
					printf("\nUnable to open file(s)\n");
					return(1);
				}
			}
			else if (usage == 2)
			{
				// files are opened by the application
				fpWav = fopen(argv[2], "rb");
				fpPac = fopen(argv[3], "wb");
				if ((fpWav == NULL) || (fpPac == NULL))
				{
					printf("\nUnable to open file(s)\n");
					return(1);
				}
				EncoderSetFiles(fpWav, fpPac);
			}

			EncoderSetVersion(version);			// file format: 2 / 4 / 5 / 6 (default)
			EncoderSetCompMode(compmode);		// compression mode: 0...5
			EncoderSetJoint(joint);				// joint stereo
			EncoderSetRA(ra);					// random access

			if (whole)	// encode file as a whole
			{
				if (EncodeFile())			// encode the whole file
				{
					printf("\nUnable to encode file\n");
					return(1);
				}
			}
			else		// encode file frame by frame
			{									// alternative: encode frame by frame
				frames = EncodeHeader();		// make pac file header, return #frames
				if (frames < 0)
				{
					printf("\nUnable to encode file\n");
					return(1);
				}

				for (f = 0; f < frames; f++)		
					EncodeFrame();				// encode a single frame

				EncodeRemains();				// read additional wave file data, generate crc
			}

			if (usage == 1)
			{
				// close files
				fclose(fpWav);
				fclose(fpPac);
			}
			else if (usage == 2)
			{
				// tell the encoder to close the files
				EncoderCloseFiles();
			}

			EncoderDelete();
		}
	}
	// decoding PAC -> WAV ////////////////////////////////////////////////////////////////////////
	else
	{
		if (usage == 0)
		{
			// decompress file.pac to file.wav using only one function call
			DecodePacFile(argv[2], argv[3]);
		}
		else
		{
			// decoder initialization
			DecoderCreate();

			if (usage == 1)
			{
				// files are opened by the decoder
				if (DecoderOpenFiles(argv[2], argv[3]))
				{
					printf("\nUnable to open file(s)\n");
					return(1);
				}
			}
			else if (usage == 2)
			{
				// files are opened by the application
				fpPac = fopen(argv[2], "wb");
				fpWav = fopen(argv[3], "rb");
				if ((fpPac == NULL) || (fpWav == NULL))
				{
					printf("\nUnable to open file(s)\n");
					return(1);
				}
				DecoderSetFiles(fpPac, fpWav);
			}

			if (whole)	// decode file as a whole
			{
				if ((crc = DecodeFile()) < 0)	// decode the whole file
				{
					printf("\nUnable to decode file\n");
					return(1);
				}
			}
			else		// decode file frame by frame
			{
				frames = DecodeHeader();		// decode pac file header, return #frames
				if (frames < 0)
				{
					printf("\nUnable to decode file\n");
					return(1);
				}

				DecoderGetInfo(&info);			// get info structure
				// todo: evaluate / display info

				short *buf = (short*)DecoderGetAudioBuffer();	// get the buffer address (suppose 16 bit)

				for (f = 0; f < frames; f++)
				{
					DecodeFrame();				// decode a single frame
					// todo: do whatever with the buffer, e.g. playback / display
					// access single samples via "buf[n]"
				}

				crc = DecodeRemains();			// write additional wave file data, return crc
			}
		}

		if (crc)
			printf("\nDECODING ERROR: CRC failed!\n");
	}

	return(0);
}  
